package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class SerpentEngine implements BlockCipher {
   private static final int BLOCK_SIZE = 16;
   static final int ROUNDS = 32;
   static final int PHI = -1640531527;
   private boolean encrypting;
   private int[] wKey;
   private int X0;
   private int X1;
   private int X2;
   private int X3;

   private int[] makeWorkingKey(byte[] key) {
      int[] var2 = new int[16];
      int var3 = 0;

      int var4;
      int var5;
      for(var4 = key.length - 4; var4 > 0; var3 = var5) {
         var5 = var3 + 1;
         var2[var3] = this.bytesToWord(key, var4);
         var4 -= 4;
      }

      if (var4 != 0) {
         throw new IllegalArgumentException("key must be a multiple of 4 bytes");
      } else {
         int var10000 = var4 = var3 + 1;
         var2[var3] = this.bytesToWord(key, 0);
         if (var10000 < 8) {
            var2[var4] = 1;
         }

         int key1 = 132;
         int[] var9 = new int[132];

         for(var4 = 8; var4 < 16; ++var4) {
            var2[var4] = this.rotateLeft(var2[var5 = var4 - 8] ^ var2[var4 - 5] ^ var2[var4 - 3] ^ var2[var4 - 1] ^ -1640531527 ^ var5, 11);
         }

         System.arraycopy(var2, 8, var9, 0, 8);

         int var8;
         for(var8 = 8; var8 < key1; ++var8) {
            var9[var8] = this.rotateLeft(var9[var8 - 8] ^ var9[var8 - 5] ^ var9[var8 - 3] ^ var9[var8 - 1] ^ -1640531527 ^ var8, 11);
         }

         int var6 = var9[0];
         key1 = var9[1];
         var8 = var9[2];
         var3 = var9[3];
         this.sb3(var6, key1, var8, var3);
         var9[0] = this.X0;
         var9[1] = this.X1;
         var9[2] = this.X2;
         var9[3] = this.X3;
         var6 = var9[4];
         key1 = var9[5];
         var8 = var9[6];
         var3 = var9[7];
         this.sb2(var6, key1, var8, var3);
         var9[4] = this.X0;
         var9[5] = this.X1;
         var9[6] = this.X2;
         var9[7] = this.X3;
         var6 = var9[8];
         key1 = var9[9];
         var8 = var9[10];
         var3 = var9[11];
         this.sb1(var6, key1, var8, var3);
         var9[8] = this.X0;
         var9[9] = this.X1;
         var9[10] = this.X2;
         var9[11] = this.X3;
         var6 = var9[12];
         key1 = var9[13];
         var8 = var9[14];
         var3 = var9[15];
         this.sb0(var6, key1, var8, var3);
         var9[12] = this.X0;
         var9[13] = this.X1;
         var9[14] = this.X2;
         var9[15] = this.X3;
         var6 = var9[16];
         key1 = var9[17];
         var8 = var9[18];
         var3 = var9[19];
         this.sb7(var6, key1, var8, var3);
         var9[16] = this.X0;
         var9[17] = this.X1;
         var9[18] = this.X2;
         var9[19] = this.X3;
         var6 = var9[20];
         key1 = var9[21];
         var8 = var9[22];
         var3 = var9[23];
         this.sb6(var6, key1, var8, var3);
         var9[20] = this.X0;
         var9[21] = this.X1;
         var9[22] = this.X2;
         var9[23] = this.X3;
         var6 = var9[24];
         key1 = var9[25];
         var8 = var9[26];
         var3 = var9[27];
         this.sb5(var6, key1, var8, var3);
         var9[24] = this.X0;
         var9[25] = this.X1;
         var9[26] = this.X2;
         var9[27] = this.X3;
         var6 = var9[28];
         key1 = var9[29];
         var8 = var9[30];
         var3 = var9[31];
         this.sb4(var6, key1, var8, var3);
         var9[28] = this.X0;
         var9[29] = this.X1;
         var9[30] = this.X2;
         var9[31] = this.X3;
         var6 = var9[32];
         key1 = var9[33];
         var8 = var9[34];
         var3 = var9[35];
         this.sb3(var6, key1, var8, var3);
         var9[32] = this.X0;
         var9[33] = this.X1;
         var9[34] = this.X2;
         var9[35] = this.X3;
         var6 = var9[36];
         key1 = var9[37];
         var8 = var9[38];
         var3 = var9[39];
         this.sb2(var6, key1, var8, var3);
         var9[36] = this.X0;
         var9[37] = this.X1;
         var9[38] = this.X2;
         var9[39] = this.X3;
         var6 = var9[40];
         key1 = var9[41];
         var8 = var9[42];
         var3 = var9[43];
         this.sb1(var6, key1, var8, var3);
         var9[40] = this.X0;
         var9[41] = this.X1;
         var9[42] = this.X2;
         var9[43] = this.X3;
         var6 = var9[44];
         key1 = var9[45];
         var8 = var9[46];
         var3 = var9[47];
         this.sb0(var6, key1, var8, var3);
         var9[44] = this.X0;
         var9[45] = this.X1;
         var9[46] = this.X2;
         var9[47] = this.X3;
         var6 = var9[48];
         key1 = var9[49];
         var8 = var9[50];
         var3 = var9[51];
         this.sb7(var6, key1, var8, var3);
         var9[48] = this.X0;
         var9[49] = this.X1;
         var9[50] = this.X2;
         var9[51] = this.X3;
         var6 = var9[52];
         key1 = var9[53];
         var8 = var9[54];
         var3 = var9[55];
         this.sb6(var6, key1, var8, var3);
         var9[52] = this.X0;
         var9[53] = this.X1;
         var9[54] = this.X2;
         var9[55] = this.X3;
         var6 = var9[56];
         key1 = var9[57];
         var8 = var9[58];
         var3 = var9[59];
         this.sb5(var6, key1, var8, var3);
         var9[56] = this.X0;
         var9[57] = this.X1;
         var9[58] = this.X2;
         var9[59] = this.X3;
         var6 = var9[60];
         key1 = var9[61];
         var8 = var9[62];
         var3 = var9[63];
         this.sb4(var6, key1, var8, var3);
         var9[60] = this.X0;
         var9[61] = this.X1;
         var9[62] = this.X2;
         var9[63] = this.X3;
         var6 = var9[64];
         key1 = var9[65];
         var8 = var9[66];
         var3 = var9[67];
         this.sb3(var6, key1, var8, var3);
         var9[64] = this.X0;
         var9[65] = this.X1;
         var9[66] = this.X2;
         var9[67] = this.X3;
         var6 = var9[68];
         key1 = var9[69];
         var8 = var9[70];
         var3 = var9[71];
         this.sb2(var6, key1, var8, var3);
         var9[68] = this.X0;
         var9[69] = this.X1;
         var9[70] = this.X2;
         var9[71] = this.X3;
         var6 = var9[72];
         key1 = var9[73];
         var8 = var9[74];
         var3 = var9[75];
         this.sb1(var6, key1, var8, var3);
         var9[72] = this.X0;
         var9[73] = this.X1;
         var9[74] = this.X2;
         var9[75] = this.X3;
         var6 = var9[76];
         key1 = var9[77];
         var8 = var9[78];
         var3 = var9[79];
         this.sb0(var6, key1, var8, var3);
         var9[76] = this.X0;
         var9[77] = this.X1;
         var9[78] = this.X2;
         var9[79] = this.X3;
         var6 = var9[80];
         key1 = var9[81];
         var8 = var9[82];
         var3 = var9[83];
         this.sb7(var6, key1, var8, var3);
         var9[80] = this.X0;
         var9[81] = this.X1;
         var9[82] = this.X2;
         var9[83] = this.X3;
         var6 = var9[84];
         key1 = var9[85];
         var8 = var9[86];
         var3 = var9[87];
         this.sb6(var6, key1, var8, var3);
         var9[84] = this.X0;
         var9[85] = this.X1;
         var9[86] = this.X2;
         var9[87] = this.X3;
         var6 = var9[88];
         key1 = var9[89];
         var8 = var9[90];
         var3 = var9[91];
         this.sb5(var6, key1, var8, var3);
         var9[88] = this.X0;
         var9[89] = this.X1;
         var9[90] = this.X2;
         var9[91] = this.X3;
         var6 = var9[92];
         key1 = var9[93];
         var8 = var9[94];
         var3 = var9[95];
         this.sb4(var6, key1, var8, var3);
         var9[92] = this.X0;
         var9[93] = this.X1;
         var9[94] = this.X2;
         var9[95] = this.X3;
         var6 = var9[96];
         key1 = var9[97];
         var8 = var9[98];
         var3 = var9[99];
         this.sb3(var6, key1, var8, var3);
         var9[96] = this.X0;
         var9[97] = this.X1;
         var9[98] = this.X2;
         var9[99] = this.X3;
         var6 = var9[100];
         key1 = var9[101];
         var8 = var9[102];
         var3 = var9[103];
         this.sb2(var6, key1, var8, var3);
         var9[100] = this.X0;
         var9[101] = this.X1;
         var9[102] = this.X2;
         var9[103] = this.X3;
         var6 = var9[104];
         key1 = var9[105];
         var8 = var9[106];
         var3 = var9[107];
         this.sb1(var6, key1, var8, var3);
         var9[104] = this.X0;
         var9[105] = this.X1;
         var9[106] = this.X2;
         var9[107] = this.X3;
         var6 = var9[108];
         key1 = var9[109];
         var8 = var9[110];
         var3 = var9[111];
         this.sb0(var6, key1, var8, var3);
         var9[108] = this.X0;
         var9[109] = this.X1;
         var9[110] = this.X2;
         var9[111] = this.X3;
         var6 = var9[112];
         key1 = var9[113];
         var8 = var9[114];
         var3 = var9[115];
         this.sb7(var6, key1, var8, var3);
         var9[112] = this.X0;
         var9[113] = this.X1;
         var9[114] = this.X2;
         var9[115] = this.X3;
         var6 = var9[116];
         key1 = var9[117];
         var8 = var9[118];
         var3 = var9[119];
         this.sb6(var6, key1, var8, var3);
         var9[116] = this.X0;
         var9[117] = this.X1;
         var9[118] = this.X2;
         var9[119] = this.X3;
         var6 = var9[120];
         key1 = var9[121];
         var8 = var9[122];
         var3 = var9[123];
         this.sb5(var6, key1, var8, var3);
         var9[120] = this.X0;
         var9[121] = this.X1;
         var9[122] = this.X2;
         var9[123] = this.X3;
         var6 = var9[124];
         key1 = var9[125];
         var8 = var9[126];
         var3 = var9[127];
         this.sb4(var6, key1, var8, var3);
         var9[124] = this.X0;
         var9[125] = this.X1;
         var9[126] = this.X2;
         var9[127] = this.X3;
         var6 = var9[128];
         key1 = var9[129];
         var8 = var9[130];
         var3 = var9[131];
         this.sb3(var6, key1, var8, var3);
         var9[128] = this.X0;
         var9[129] = this.X1;
         var9[130] = this.X2;
         var9[131] = this.X3;
         return var9;
      }
   }

   private int rotateLeft(int x, int bits) {
      return x << bits | x >>> -bits;
   }

   private int rotateRight(int x, int bits) {
      return x >>> bits | x << -bits;
   }

   private int bytesToWord(byte[] src, int srcOff) {
      return (src[srcOff] & 255) << 24 | (src[srcOff + 1] & 255) << 16 | (src[srcOff + 2] & 255) << 8 | src[srcOff + 3] & 255;
   }

   private void wordToBytes(int word, byte[] dst, int dstOff) {
      int var4 = dstOff + 3;
      dst[var4] = (byte)word;
      var4 = dstOff + 2;
      dst[var4] = (byte)(word >>> 8);
      var4 = dstOff + 1;
      dst[var4] = (byte)(word >>> 16);
      dst[dstOff] = (byte)(word >>> 24);
   }

   private void encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      this.X3 = this.bytesToWord(in, inOff);
      this.X2 = this.bytesToWord(in, inOff + 4);
      this.X1 = this.bytesToWord(in, inOff + 8);
      int in1;
      this.X0 = in1 = this.bytesToWord(in, inOff + 12);
      int[] var10102 = this.wKey;
      in1 ^= var10102[0];
      inOff = var10102[1] ^ this.X1;
      int var5 = var10102[2] ^ this.X2;
      int var6 = var10102[3] ^ this.X3;
      this.sb0(in1, inOff, var5, var6);
      this.LT();
      int[] var10099 = this.wKey;
      in1 = var10099[4] ^ this.X0;
      inOff = var10099[5] ^ this.X1;
      var5 = var10099[6] ^ this.X2;
      var6 = var10099[7] ^ this.X3;
      this.sb1(in1, inOff, var5, var6);
      this.LT();
      int[] var10096 = this.wKey;
      in1 = var10096[8] ^ this.X0;
      inOff = var10096[9] ^ this.X1;
      var5 = var10096[10] ^ this.X2;
      var6 = var10096[11] ^ this.X3;
      this.sb2(in1, inOff, var5, var6);
      this.LT();
      int[] var10093 = this.wKey;
      in1 = var10093[12] ^ this.X0;
      inOff = var10093[13] ^ this.X1;
      var5 = var10093[14] ^ this.X2;
      var6 = var10093[15] ^ this.X3;
      this.sb3(in1, inOff, var5, var6);
      this.LT();
      int[] var10090 = this.wKey;
      in1 = var10090[16] ^ this.X0;
      inOff = var10090[17] ^ this.X1;
      var5 = var10090[18] ^ this.X2;
      var6 = var10090[19] ^ this.X3;
      this.sb4(in1, inOff, var5, var6);
      this.LT();
      int[] var10087 = this.wKey;
      in1 = var10087[20] ^ this.X0;
      inOff = var10087[21] ^ this.X1;
      var5 = var10087[22] ^ this.X2;
      var6 = var10087[23] ^ this.X3;
      this.sb5(in1, inOff, var5, var6);
      this.LT();
      int[] var10084 = this.wKey;
      in1 = var10084[24] ^ this.X0;
      inOff = var10084[25] ^ this.X1;
      var5 = var10084[26] ^ this.X2;
      var6 = var10084[27] ^ this.X3;
      this.sb6(in1, inOff, var5, var6);
      this.LT();
      int[] var10081 = this.wKey;
      in1 = var10081[28] ^ this.X0;
      inOff = var10081[29] ^ this.X1;
      var5 = var10081[30] ^ this.X2;
      var6 = var10081[31] ^ this.X3;
      this.sb7(in1, inOff, var5, var6);
      this.LT();
      int[] var10078 = this.wKey;
      in1 = var10078[32] ^ this.X0;
      inOff = var10078[33] ^ this.X1;
      var5 = var10078[34] ^ this.X2;
      var6 = var10078[35] ^ this.X3;
      this.sb0(in1, inOff, var5, var6);
      this.LT();
      int[] var10075 = this.wKey;
      in1 = var10075[36] ^ this.X0;
      inOff = var10075[37] ^ this.X1;
      var5 = var10075[38] ^ this.X2;
      var6 = var10075[39] ^ this.X3;
      this.sb1(in1, inOff, var5, var6);
      this.LT();
      int[] var10072 = this.wKey;
      in1 = var10072[40] ^ this.X0;
      inOff = var10072[41] ^ this.X1;
      var5 = var10072[42] ^ this.X2;
      var6 = var10072[43] ^ this.X3;
      this.sb2(in1, inOff, var5, var6);
      this.LT();
      int[] var10069 = this.wKey;
      in1 = var10069[44] ^ this.X0;
      inOff = var10069[45] ^ this.X1;
      var5 = var10069[46] ^ this.X2;
      var6 = var10069[47] ^ this.X3;
      this.sb3(in1, inOff, var5, var6);
      this.LT();
      int[] var10066 = this.wKey;
      in1 = var10066[48] ^ this.X0;
      inOff = var10066[49] ^ this.X1;
      var5 = var10066[50] ^ this.X2;
      var6 = var10066[51] ^ this.X3;
      this.sb4(in1, inOff, var5, var6);
      this.LT();
      int[] var10063 = this.wKey;
      in1 = var10063[52] ^ this.X0;
      inOff = var10063[53] ^ this.X1;
      var5 = var10063[54] ^ this.X2;
      var6 = var10063[55] ^ this.X3;
      this.sb5(in1, inOff, var5, var6);
      this.LT();
      int[] var10060 = this.wKey;
      in1 = var10060[56] ^ this.X0;
      inOff = var10060[57] ^ this.X1;
      var5 = var10060[58] ^ this.X2;
      var6 = var10060[59] ^ this.X3;
      this.sb6(in1, inOff, var5, var6);
      this.LT();
      int[] var10057 = this.wKey;
      in1 = var10057[60] ^ this.X0;
      inOff = var10057[61] ^ this.X1;
      var5 = var10057[62] ^ this.X2;
      var6 = var10057[63] ^ this.X3;
      this.sb7(in1, inOff, var5, var6);
      this.LT();
      int[] var10054 = this.wKey;
      in1 = var10054[64] ^ this.X0;
      inOff = var10054[65] ^ this.X1;
      var5 = var10054[66] ^ this.X2;
      var6 = var10054[67] ^ this.X3;
      this.sb0(in1, inOff, var5, var6);
      this.LT();
      int[] var10051 = this.wKey;
      in1 = var10051[68] ^ this.X0;
      inOff = var10051[69] ^ this.X1;
      var5 = var10051[70] ^ this.X2;
      var6 = var10051[71] ^ this.X3;
      this.sb1(in1, inOff, var5, var6);
      this.LT();
      int[] var10048 = this.wKey;
      in1 = var10048[72] ^ this.X0;
      inOff = var10048[73] ^ this.X1;
      var5 = var10048[74] ^ this.X2;
      var6 = var10048[75] ^ this.X3;
      this.sb2(in1, inOff, var5, var6);
      this.LT();
      int[] var10045 = this.wKey;
      in1 = var10045[76] ^ this.X0;
      inOff = var10045[77] ^ this.X1;
      var5 = var10045[78] ^ this.X2;
      var6 = var10045[79] ^ this.X3;
      this.sb3(in1, inOff, var5, var6);
      this.LT();
      int[] var10042 = this.wKey;
      in1 = var10042[80] ^ this.X0;
      inOff = var10042[81] ^ this.X1;
      var5 = var10042[82] ^ this.X2;
      var6 = var10042[83] ^ this.X3;
      this.sb4(in1, inOff, var5, var6);
      this.LT();
      int[] var10039 = this.wKey;
      in1 = var10039[84] ^ this.X0;
      inOff = var10039[85] ^ this.X1;
      var5 = var10039[86] ^ this.X2;
      var6 = var10039[87] ^ this.X3;
      this.sb5(in1, inOff, var5, var6);
      this.LT();
      int[] var10036 = this.wKey;
      in1 = var10036[88] ^ this.X0;
      inOff = var10036[89] ^ this.X1;
      var5 = var10036[90] ^ this.X2;
      var6 = var10036[91] ^ this.X3;
      this.sb6(in1, inOff, var5, var6);
      this.LT();
      int[] var10033 = this.wKey;
      in1 = var10033[92] ^ this.X0;
      inOff = var10033[93] ^ this.X1;
      var5 = var10033[94] ^ this.X2;
      var6 = var10033[95] ^ this.X3;
      this.sb7(in1, inOff, var5, var6);
      this.LT();
      int[] var10030 = this.wKey;
      in1 = var10030[96] ^ this.X0;
      inOff = var10030[97] ^ this.X1;
      var5 = var10030[98] ^ this.X2;
      var6 = var10030[99] ^ this.X3;
      this.sb0(in1, inOff, var5, var6);
      this.LT();
      int[] var10027 = this.wKey;
      in1 = var10027[100] ^ this.X0;
      inOff = var10027[101] ^ this.X1;
      var5 = var10027[102] ^ this.X2;
      var6 = var10027[103] ^ this.X3;
      this.sb1(in1, inOff, var5, var6);
      this.LT();
      int[] var10024 = this.wKey;
      in1 = var10024[104] ^ this.X0;
      inOff = var10024[105] ^ this.X1;
      var5 = var10024[106] ^ this.X2;
      var6 = var10024[107] ^ this.X3;
      this.sb2(in1, inOff, var5, var6);
      this.LT();
      int[] var10021 = this.wKey;
      in1 = var10021[108] ^ this.X0;
      inOff = var10021[109] ^ this.X1;
      var5 = var10021[110] ^ this.X2;
      var6 = var10021[111] ^ this.X3;
      this.sb3(in1, inOff, var5, var6);
      this.LT();
      int[] var10018 = this.wKey;
      in1 = var10018[112] ^ this.X0;
      inOff = var10018[113] ^ this.X1;
      var5 = var10018[114] ^ this.X2;
      var6 = var10018[115] ^ this.X3;
      this.sb4(in1, inOff, var5, var6);
      this.LT();
      int[] var10015 = this.wKey;
      in1 = var10015[116] ^ this.X0;
      inOff = var10015[117] ^ this.X1;
      var5 = var10015[118] ^ this.X2;
      var6 = var10015[119] ^ this.X3;
      this.sb5(in1, inOff, var5, var6);
      this.LT();
      int[] var10012 = this.wKey;
      in1 = var10012[120] ^ this.X0;
      inOff = var10012[121] ^ this.X1;
      var5 = var10012[122] ^ this.X2;
      var6 = var10012[123] ^ this.X3;
      this.sb6(in1, inOff, var5, var6);
      this.LT();
      int[] var10009 = this.wKey;
      in1 = var10009[124] ^ this.X0;
      inOff = var10009[125] ^ this.X1;
      var5 = var10009[126] ^ this.X2;
      var6 = var10009[127] ^ this.X3;
      this.sb7(in1, inOff, var5, var6);
      this.wordToBytes(this.wKey[131] ^ this.X3, out, outOff);
      this.wordToBytes(this.wKey[130] ^ this.X2, out, outOff + 4);
      this.wordToBytes(this.wKey[129] ^ this.X1, out, outOff + 8);
      this.wordToBytes(this.wKey[128] ^ this.X0, out, outOff + 12);
   }

   private void decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      this.X3 = this.wKey[131] ^ this.bytesToWord(in, inOff);
      this.X2 = this.wKey[130] ^ this.bytesToWord(in, inOff + 4);
      this.X1 = this.wKey[129] ^ this.bytesToWord(in, inOff + 8);
      int in1;
      this.X0 = in1 = this.wKey[128] ^ this.bytesToWord(in, inOff + 12);
      inOff = this.X1;
      int var5 = this.X2;
      int var6 = this.X3;
      this.ib7(in1, inOff, var5, var6);
      int[] in2;
      this.X0 ^= (in2 = this.wKey)[124];
      this.X1 ^= in2[125];
      this.X2 ^= in2[126];
      this.X3 ^= in2[127];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib6(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[120];
      this.X1 ^= in2[121];
      this.X2 ^= in2[122];
      this.X3 ^= in2[123];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib5(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[116];
      this.X1 ^= in2[117];
      this.X2 ^= in2[118];
      this.X3 ^= in2[119];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib4(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[112];
      this.X1 ^= in2[113];
      this.X2 ^= in2[114];
      this.X3 ^= in2[115];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib3(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[108];
      this.X1 ^= in2[109];
      this.X2 ^= in2[110];
      this.X3 ^= in2[111];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib2(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[104];
      this.X1 ^= in2[105];
      this.X2 ^= in2[106];
      this.X3 ^= in2[107];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib1(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[100];
      this.X1 ^= in2[101];
      this.X2 ^= in2[102];
      this.X3 ^= in2[103];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib0(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[96];
      this.X1 ^= in2[97];
      this.X2 ^= in2[98];
      this.X3 ^= in2[99];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib7(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[92];
      this.X1 ^= in2[93];
      this.X2 ^= in2[94];
      this.X3 ^= in2[95];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib6(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[88];
      this.X1 ^= in2[89];
      this.X2 ^= in2[90];
      this.X3 ^= in2[91];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib5(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[84];
      this.X1 ^= in2[85];
      this.X2 ^= in2[86];
      this.X3 ^= in2[87];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib4(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[80];
      this.X1 ^= in2[81];
      this.X2 ^= in2[82];
      this.X3 ^= in2[83];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib3(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[76];
      this.X1 ^= in2[77];
      this.X2 ^= in2[78];
      this.X3 ^= in2[79];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib2(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[72];
      this.X1 ^= in2[73];
      this.X2 ^= in2[74];
      this.X3 ^= in2[75];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib1(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[68];
      this.X1 ^= in2[69];
      this.X2 ^= in2[70];
      this.X3 ^= in2[71];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib0(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[64];
      this.X1 ^= in2[65];
      this.X2 ^= in2[66];
      this.X3 ^= in2[67];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib7(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[60];
      this.X1 ^= in2[61];
      this.X2 ^= in2[62];
      this.X3 ^= in2[63];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib6(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[56];
      this.X1 ^= in2[57];
      this.X2 ^= in2[58];
      this.X3 ^= in2[59];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib5(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[52];
      this.X1 ^= in2[53];
      this.X2 ^= in2[54];
      this.X3 ^= in2[55];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib4(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[48];
      this.X1 ^= in2[49];
      this.X2 ^= in2[50];
      this.X3 ^= in2[51];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib3(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[44];
      this.X1 ^= in2[45];
      this.X2 ^= in2[46];
      this.X3 ^= in2[47];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib2(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[40];
      this.X1 ^= in2[41];
      this.X2 ^= in2[42];
      this.X3 ^= in2[43];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib1(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[36];
      this.X1 ^= in2[37];
      this.X2 ^= in2[38];
      this.X3 ^= in2[39];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib0(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[32];
      this.X1 ^= in2[33];
      this.X2 ^= in2[34];
      this.X3 ^= in2[35];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib7(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[28];
      this.X1 ^= in2[29];
      this.X2 ^= in2[30];
      this.X3 ^= in2[31];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib6(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[24];
      this.X1 ^= in2[25];
      this.X2 ^= in2[26];
      this.X3 ^= in2[27];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib5(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[20];
      this.X1 ^= in2[21];
      this.X2 ^= in2[22];
      this.X3 ^= in2[23];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib4(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[16];
      this.X1 ^= in2[17];
      this.X2 ^= in2[18];
      this.X3 ^= in2[19];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib3(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[12];
      this.X1 ^= in2[13];
      this.X2 ^= in2[14];
      this.X3 ^= in2[15];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib2(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[8];
      this.X1 ^= in2[9];
      this.X2 ^= in2[10];
      this.X3 ^= in2[11];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib1(in1, inOff, var5, var6);
      this.X0 ^= (in2 = this.wKey)[4];
      this.X1 ^= in2[5];
      this.X2 ^= in2[6];
      this.X3 ^= in2[7];
      this.inverseLT();
      in1 = this.X0;
      inOff = this.X1;
      var5 = this.X2;
      var6 = this.X3;
      this.ib0(in1, inOff, var5, var6);
      this.wordToBytes(this.X3 ^ this.wKey[3], out, outOff);
      this.wordToBytes(this.X2 ^ this.wKey[2], out, outOff + 4);
      this.wordToBytes(this.X1 ^ this.wKey[1], out, outOff + 8);
      this.wordToBytes(this.X0 ^ this.wKey[0], out, outOff + 12);
   }

   private void sb0(int a, int b, int c, int d) {
      int var10002 = d;
      int var5;
      int var6 = b ^ (var5 = c ^ (d ^= a));
      int var7;
      int var10001 = var7 = a & var10002 ^ var6;
      this.X3 = var7;
      this.X2 = var6 ^ (c | (a ^= b & d));
      var10001 = b = var10001 & (var5 ^ a);
      this.X1 = ~var5 ^ b;
      this.X0 = var10001 ^ ~a;
   }

   private void ib0(int a, int b, int c, int d) {
      int var10001 = a;
      int var5;
      int var10003 = b = d ^ ((var5 = ~a) | (a ^= b));
      int var6;
      int var10004 = var6 = a ^ (c ^= b);
      this.X2 = var6;
      this.X1 = d = var10003 ^ var10004 & (a = var5 ^ d & a);
      var10001 = b = var10001 & b ^ (c | d);
      this.X3 = b;
      this.X0 = var10001 ^ c ^ a;
   }

   private void sb1(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = c ^= a | (a = b ^ ~a);
      int var10002 = a;
      int var10005 = a;
      this.X2 = a = d ^ c;
      b ^= d | var10005;
      var10002 ^= a;
      int var10004 = a = var10002 ^ c & b;
      this.X3 = a;
      int var5;
      this.X1 = var10004 ^ (var5 = c ^ b);
      var10000.X0 = var10001 ^ var10002 & var5;
   }

   private void ib1(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var5;
      int var10001 = var5 = (d ^= b) ^ (a ^= b & d);
      int var10003 = c ^= var5;
      this.X3 = c;
      int var10002 = ~(this.X1 = a ^ (var10003 | (a = b ^ d & a)));
      int var6;
      this.X0 = var10002 ^ (var6 = c ^ a);
      var10000.X2 = var10001 ^ (var10002 | var6);
   }

   private void sb2(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var5 = ~a;
      int var6;
      int var10001 = var6 = b ^ d;
      SerpentEngine var10002 = this;
      int var10003 = a;
      SerpentEngine var10005 = this;
      int var7;
      this.X0 = var7 = var6 ^ c & var5;
      var10005.X3 = c = (a = c ^ var5) ^ (b &= c ^ var7);
      var10002.X2 = var7 = var10003 ^ (d | b) & (var7 | a);
      var10000.X1 = var10001 ^ c ^ var7 ^ (d | var5);
   }

   private void ib2(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      SerpentEngine var10002 = this;
      int var10003 = c;
      int var10005 = c;
      int var5 = ~(c = b ^ d);
      int var6 = a ^ var10005;
      int var7;
      var10003 = var7 = var10003 ^ c;
      SerpentEngine var10004 = this;
      int var8;
      this.X0 = var8 = var6 ^ b & var7;
      var10004.X3 = a = c ^ (var6 | d ^ (a | var5));
      var10002.X1 = (b = ~var10003) ^ (var8 |= a);
      var10000.X2 = d & b ^ var6 ^ var8;
   }

   private void sb3(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = b;
      int var10002 = d;
      SerpentEngine var10003 = this;
      int var5 = a ^ b;
      int var10004 = a & c;
      a |= d;
      c ^= d;
      var10004 = d = var10004 | var5 & a;
      int var10006 = a;
      this.X2 = a = c ^ d;
      var10004 = b = var10004 ^ b ^ var10006;
      int var6;
      this.X0 = var6 = var5 ^ c & b;
      var10003.X1 = var10004 ^ (var6 &= a);
      var10000.X3 = (var10001 | var10002) ^ c ^ var6;
   }

   private void ib3(int a, int b, int c, int d) {
      int var10001 = a;
      a |= b;
      int var5;
      var10001 = var5 = var10001 ^ b & (b ^= c);
      c ^= var5;
      int var6;
      int var10002 = var6 = b ^ (var5 |= d);
      int var10003 = a;
      this.X0 = var6;
      this.X2 = c ^ (a = d ^ (b | var5));
      var10001 = b = var10001 ^ var10002 & (a ^= var10003);
      this.X3 = b;
      this.X1 = var10001 ^ var6 ^ a;
   }

   private void sb4(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = a;
      SerpentEngine var10002 = this;
      int var10003 = a;
      SerpentEngine var10004 = this;
      int var10005 = c ^= d & (a ^= d);
      int var5;
      this.X3 = a ^ (var5 = b | c);
      var10004.X0 = d = var10005 ^ (a | (b = ~b));
      var10002.X2 = a = var10003 & d ^ var5 & (var5 = a ^ b);
      var10000.X1 = var10001 ^ c ^ var5 & a;
   }

   private void ib4(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = b ^= a & (c | d);
      int var10003 = b;
      int var10004 = b = c ^ a & b;
      int var10005 = a;
      this.X1 = a = d ^ b;
      b = ~var10005;
      var10003 = c = var10003 ^ var10004 & a;
      this.X3 = c;
      int var5;
      this.X0 = var10003 ^ (var5 = d ^ (a | b));
      var10000.X2 = var10001 & var5 ^ a ^ b;
   }

   private void sb5(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = b;
      SerpentEngine var10002 = this;
      int var10004 = b;
      b = ~a;
      int var5;
      int var10003 = var5 = a ^ var10004;
      var10004 = b;
      SerpentEngine var10005 = this;
      SerpentEngine var10007 = this;
      int var6 = a ^ d;
      var10007.X0 = a = c ^ b ^ (var5 | var6);
      var10005.X1 = c = (b = d & a) ^ var5 ^ a;
      a |= var10004;
      var10002.X2 = (var10003 | b) ^ (var6 ^= a);
      var10000.X3 = var10001 ^ b ^ c & var6;
   }

   private void ib5(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = b;
      SerpentEngine var10002 = this;
      int var10004 = d;
      SerpentEngine var10005 = this;
      int var5;
      int var10006 = var5 = d ^ b & (d = ~c);
      int var10008 = b;
      int var6;
      this.X3 = b = (var6 = a & var5) ^ b ^ d;
      var10005.X1 = var10006 ^ a & (b |= var10008);
      var10002.X0 = (var5 = a | var10004) ^ d ^ b;
      var10000.X2 = var10001 & var5 ^ (var6 | a ^ c);
   }

   private void sb6(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = c;
      int var10002 = ~a;
      c = b ^ (a ^= d);
      int var5;
      var10001 = var5 = var10001 ^ (var10002 | a);
      int var10006 = a;
      this.X1 = a = b ^ var5;
      int var10003 = b = c ^ var5 & (a = d ^ (var10006 | a));
      this.X2 = b;
      int var6;
      this.X0 = var10003 ^ (var6 = var5 ^ a);
      var10000.X3 = ~var10001 ^ c & var6;
   }

   private void ib6(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var10001 = d;
      SerpentEngine var10002 = this;
      int var10003 = a;
      a = ~a;
      var10003 ^= b;
      int var5;
      int var10004 = var5 = c ^ var10003;
      this.X1 = var5 ^ (c = d ^ (c | a));
      var10003 = d = var10003 ^ var10004 & c;
      int var6;
      this.X3 = var6 = c ^ (b | d);
      var10002.X0 = var10003 ^ (var6 |= b);
      var10000.X2 = var10001 & a ^ var5 ^ var6;
   }

   private void sb7(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var5;
      int var6;
      int var10001 = var6 = a ^ (var5 = d ^ c & (c ^= b));
      int var10004 = a;
      this.X1 = a = b ^ var6 & (d | c);
      a |= var5;
      this.X3 = b = c ^ var10004 & var6;
      var10001 = a ^= var10001;
      int var7;
      this.X2 = var7 = var5 ^ b & a;
      var10000.X0 = ~var10001 ^ b & var7;
   }

   private void ib7(int a, int b, int c, int d) {
      SerpentEngine var10000 = this;
      int var5;
      int var10001 = var5 = c | a & b;
      SerpentEngine var10002 = this;
      int var10003 = c;
      SerpentEngine var10004 = this;
      int var10006 = b;
      int var6;
      this.X3 = b = var5 ^ (var6 = d & (a | b));
      c = ~d;
      var10004.X1 = c = a ^ ((var6 ^= var10006) | b ^ c);
      var10002.X0 = var6 = var10003 ^ var6 ^ (d | c);
      var10000.X2 = var10001 ^ c ^ var6 ^ a & b;
   }

   private void LT() {
      int var1 = this.rotateLeft(this.X0, 13);
      int var2;
      int var10002 = var2 = this.rotateLeft(this.X2, 3);
      int var10006 = var1;
      int var10012 = this.X1 ^ var1 ^ var2;
      var1 = this.X3 ^ var2 ^ var1 << 3;
      this.X1 = this.rotateLeft(var10012, 1);
      this.X3 = var1 = this.rotateLeft(var1, 7);
      this.X0 = this.rotateLeft(var10006 ^ this.X1 ^ var1, 5);
      this.X2 = this.rotateLeft(var10002 ^ this.X3 ^ this.X1 << 7, 22);
   }

   private void inverseLT() {
      int var1 = this.rotateRight(this.X2, 22) ^ this.X3 ^ this.X1 << 7;
      int var2;
      int var3 = this.rotateRight(this.X0, 5) ^ this.X1 ^ (var2 = this.X3);
      var2 = this.rotateRight(var2, 7);
      int var10005 = this.rotateRight(this.X1, 1);
      this.X3 = var2 ^ var1 ^ var3 << 3;
      this.X1 = var10005 ^ var3 ^ var1;
      this.X2 = this.rotateRight(var1, 3);
      this.X0 = this.rotateRight(var3, 13);
   }

   public void init(boolean encrypting, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this.encrypting = encrypting;
         this.wKey = this.makeWorkingKey(((KeyParameter)params).getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to Serpent init - " + params.getClass().getName());
      }
   }

   public String getAlgorithmName() {
      return "Serpent";
   }

   public int getBlockSize() {
      return 16;
   }

   public final int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.wKey != null) {
         if (inOff + 16 <= in.length) {
            if (outOff + 16 <= out.length) {
               if (this.encrypting) {
                  this.encryptBlock(in, inOff, out, outOff);
               } else {
                  this.decryptBlock(in, inOff, out, outOff);
               }

               return 16;
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("Serpent not initialised");
      }
   }

   public void reset() {
   }
}
